@@ -21,8 +21,6 @@ class cmGeneratedFileStream;
 
 /** \class cmExtraEclipseCDT4Generator
  * \brief Write Eclipse project files for Makefile based projects
- *
- * This generator is in early alpha stage.
  */
 class cmExtraEclipseCDT4Generator : public cmExternalMakefileProjectGenerator
 {
@@ -42,8 +40,6 @@ public:
   virtual void GetDocumentation(cmDocumentationEntry& entry,
                                 const char*           fullName) const;
 
-  virtual void SetGlobalGenerator(cmGlobalGenerator* generator);
-
   virtual void Generate();
 
 private:
@@ -56,18 +52,6 @@ private:
   // create .cproject file
   void CreateCProjectFile() const;
 
-  // Eclipse supported toolchain types
-  enum EclipseToolchainType
-    {
-    EclipseToolchainOther,
-    EclipseToolchainLinux,
-    EclipseToolchainCygwin,
-    EclipseToolchainMinGW,
-    EclipseToolchainSolaris,
-    EclipseToolchainMacOSX
-    };
-  static EclipseToolchainType GetToolChainType(const cmMakefile& makefile);
-
   // If built with cygwin cmake, convert posix to windows path.
   static std::string GetEclipsePath(const std::string& path);
 
@@ -82,11 +66,12 @@ private:
   static std::string EscapeForXML(const std::string& value);
 
   // Helper functions
-  static void AppendStorageScanners(cmGeneratedFileStream& fout, 
+  static void AppendStorageScanners(cmGeneratedFileStream& fout,
                                     const cmMakefile& makefile);
   static void AppendTarget         (cmGeneratedFileStream& fout,
                                     const std::string&     target,
                                     const std::string&     make,
+                                    const std::string&     makeArguments,
                                     const std::string&     path,
                                     const char* prefix = "");
   static void AppendScannerProfile (cmGeneratedFileStream& fout,
@@ -102,7 +87,8 @@ private:
 
   static void AppendLinkedResource (cmGeneratedFileStream& fout,
                                     const std::string&     name,
-                                    const std::string&     path);
+                                    const std::string&     path,
+                                    bool isVirtualFolder = false);
 
   bool AppendOutLinkedResource(cmGeneratedFileStream& fout,
                                const std::string&     defname,
@@ -112,6 +98,9 @@ private:
                                    const std::vector<std::string>& includeDirs,
                                    std::set<std::string>& emittedDirs);
 
+  static void AddEnvVar(cmGeneratedFileStream& fout, const char* envVar,
+                        cmMakefile* mf);
+
   std::vector<std::string> SrcLinkedResources;
   std::vector<std::string> OutLinkedResources;
   std::string HomeDirectory;
