@@ -14,6 +14,7 @@
 #include "cmMakefile.h"
 #include "cmSourceFile.h"
 #include "cmSystemTools.h"
+#include "cmCustomCommandGenerator.h"
 #include "windows.h"
 
 //----------------------------------------------------------------------------
@@ -31,13 +32,17 @@ cmLocalVisualStudioGenerator::~cmLocalVisualStudioGenerator()
 //----------------------------------------------------------------------------
 cmsys::auto_ptr<cmCustomCommand>
 cmLocalVisualStudioGenerator::MaybeCreateImplibDir(cmTarget& target,
-                                                   const char* config)
+                                                   const char* config,
+                                                   bool isFortran)
 {
   cmsys::auto_ptr<cmCustomCommand> pcc;
 
   // If an executable exports symbols then VS wants to create an
   // import library but forgets to create the output directory.
-  if(target.GetType() != cmTarget::EXECUTABLE) { return pcc; }
+  // The Intel Fortran plugin always forgets to the directory.
+  if(target.GetType() != cmTarget::EXECUTABLE &&
+     !(isFortran && target.GetType() == cmTarget::SHARED_LIBRARY))
+    { return pcc; }
   std::string outDir = target.GetDirectory(config, false);
   std::string impDir = target.GetDirectory(config, true);
   if(impDir == outDir) { return pcc; }
@@ -52,7 +57,7 @@ cmLocalVisualStudioGenerator::MaybeCreateImplibDir(cmTarget& target,
   std::vector<std::string> no_depends;
   cmCustomCommandLines commands;
   commands.push_back(command);
-  pcc.reset(new cmCustomCommand(no_output, no_depends, commands, 0, 0));
+  pcc.reset(new cmCustomCommand(0, no_output, no_depends, commands, 0, 0));
   pcc->SetEscapeOldStyle(false);
   pcc->SetEscapeAllowMakeVars(true);
   return pcc;
@@ -148,21 +153,57 @@ void cmLocalVisualStudioGenerator::ComputeObjectNameRequirements
   this->InsertNeedObjectNames(sourceGroups, objectNameCounts);
 }
 
+//----------------------------------------------------------------------------
+const char* cmLocalVisualStudioGenerator::ReportErrorLabel() const
+{
+  return ":VCReportError";
+}
+
+//----------------------------------------------------------------------------
+const char* cmLocalVisualStudioGenerator::GetReportErrorLabel() const
+{
+  return this->ReportErrorLabel();
+}
+
 //----------------------------------------------------------------------------
 std::string
 cmLocalVisualStudioGenerator
-::ConstructScript(const cmCustomCommandLines& commandLines,
-                  const char* workingDirectory,
+::ConstructScript(cmCustomCommand const& cc,
                   const char* configName,
-                  bool escapeOldStyle,
-                  bool escapeAllowMakeVars,
                   const char* newline_text)
 {
+  bool useLocal = this->CustomCommandUseLocal();
+  const cmCustomCommandLines& commandLines = cc.GetCommandLines();
+  const char* workingDirectory = cc.GetWorkingDirectory();
+  cmCustomCommandGenerator ccg(cc, configName, this->Makefile);
+  RelativeRoot relativeRoot = workingDirectory? NONE : START_OUTPUT;
+
   // Avoid leading or trailing newlines.
   const char* newline = "";
 
+  // Line to check for error between commands.
+  std::string check_error = newline_text;
+  if(useLocal)
+    {
+    check_error += "if %errorlevel% neq 0 goto :cmEnd";
+    }
+  else
+    {
+    check_error += "if errorlevel 1 goto ";
+    check_error += this->GetReportErrorLabel();
+    }
+
   // Store the script in a string.
   std::string script;
+
+  // Open a local context.
+  if(useLocal)
+    {
+    script += newline;
+    newline = newline_text;
+    script += "setlocal";
+    }
+
   if(workingDirectory)
     {
     // Change the working directory.
@@ -170,6 +211,7 @@ cmLocalVisualStudioGenerator
     newline = newline_text;
     script += "cd ";
     script += this->Convert(workingDirectory, FULL, SHELL);
+    script += check_error;
 
     // Change the working drive.
     if(workingDirectory[0] && workingDirectory[1] == ':')
@@ -178,8 +220,10 @@ cmLocalVisualStudioGenerator
       newline = newline_text;
       script += workingDirectory[0];
       script += workingDirectory[1];
+      script += check_error;
       }
     }
+
   // for visual studio IDE add extra stuff to the PATH
   // if CMAKE_MSVCIDE_RUN_PATH is set.
   if(this->Makefile->GetDefinition("MSVC_IDE"))
@@ -195,49 +239,42 @@ cmLocalVisualStudioGenerator
       script += ";%PATH%";
       }
     }
+
   // Write each command on a single line.
-  for(cmCustomCommandLines::const_iterator cl = commandLines.begin();
-      cl != commandLines.end(); ++cl)
+  for(unsigned int c = 0; c < ccg.GetNumberOfCommands(); ++c)
     {
     // Start a new line.
     script += newline;
     newline = newline_text;
 
-    // Start with the command name.
-    const cmCustomCommandLine& commandLine = *cl;
-    std::string commandName = this->GetRealLocation(commandLine[0].c_str(), 
-                                                    configName);
-    if(!workingDirectory)
-      {
-      script += this->Convert(commandName.c_str(),START_OUTPUT,SHELL);
-      }
-    else
-      {
-      script += this->Convert(commandName.c_str(),NONE,SHELL);
-      }
-
-    // Add the arguments.
-    for(unsigned int j=1;j < commandLine.size(); ++j)
-      {
-      script += " ";
-      if(escapeOldStyle)
-        {
-        script += this->EscapeForShellOldStyle(commandLine[j].c_str());
-        }
-      else
-        {
-        script += this->EscapeForShell(commandLine[j].c_str(),
-                                       escapeAllowMakeVars);
-        }
-      }
+    // Add this command line.
+    std::string cmd = ccg.GetCommand(c);
+    script += this->Convert(cmd.c_str(), relativeRoot, SHELL);
+    ccg.AppendArguments(c, script);
 
     // After each custom command, check for an error result.
     // If there was an error, jump to the VCReportError label,
     // skipping the run of any subsequent commands in this
     // sequence.
-    //
-    script += newline_text;
-    script += "if errorlevel 1 goto VCReportError";
+    script += check_error;
+    }
+
+  // Close the local context.
+  if(useLocal)
+    {
+    script += newline;
+    script += ":cmEnd";
+    script += newline;
+    script += "endlocal & call :cmErrorLevel %errorlevel% & goto :cmDone";
+    script += newline;
+    script += ":cmErrorLevel";
+    script += newline;
+    script += "exit /b %1";
+    script += newline;
+    script += ":cmDone";
+    script += newline;
+    script += "if %errorlevel% neq 0 goto ";
+    script += this->GetReportErrorLabel();
     }
 
   return script;
