@@ -61,6 +61,14 @@ public:
   unsigned int GetCacheMajorVersion();
   unsigned int GetCacheMinorVersion();
 
+  /* Check for unused variables in this scope */
+  void CheckForUnusedVariables() const;
+  /* Mark a variable as used */
+  void MarkVariableAsUsed(const char* var);
+  /* return true if a variable has been initialized */
+  bool VariableInitialized(const char* ) const;
+  /* return true if a variable has been used */
+  bool VariableUsed(const char* ) const;
   /** Return whether compatibility features needed for a version of
       the cache or lower should be enabled.  */
   bool NeedCacheCompatibility(int major, int minor);
@@ -309,7 +317,11 @@ public:
     {
       return this->ProjectName.c_str();
     }
-  
+
+  /** Get the configurations to be generated.  */
+  const char* GetConfigurations(std::vector<std::string>& configs,
+                                bool single = true) const;
+
   /**
    * Set the name of the library.
    */
@@ -399,7 +411,17 @@ public:
       return this->HomeOutputDirectory.c_str();
     }
   //@}
-  
+
+  /**
+   * Set CMAKE_SCRIPT_MODE_FILE variable when running a -P script.
+   */
+  void SetScriptModeFile(const char* scriptfile);
+
+  /**
+   * Set CMAKE_ARGC, CMAKE_ARGV0 ... variables.
+   */
+  void SetArgcArgv(const std::vector<std::string>& args);
+
   //@{
   /**
    * Set/Get the start directory (or output directory). The start directory
@@ -832,7 +854,10 @@ public:
 protected:
   // add link libraries and directories to the target
   void AddGlobalLinkInformation(const char* name, cmTarget& target);
-  
+
+  // Check for a an unused variable
+  void CheckForUnused(const char* reason, const char* name) const;
+
   std::string Prefix;
   std::vector<std::string> AuxSourceDirectories; // 
 
@@ -925,6 +950,10 @@ private:
   // should this makefile be processed before or after processing the parent
   bool PreOrder;
 
+  // Unused variable flags
+  bool WarnUnused;
+  bool CheckSystemVars;
+
   // stack of list files being read 
   std::deque<cmStdString> ListFileStack;
 
