@@ -1,19 +1,14 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmSetSourceFilesPropertiesCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2008-02-15 16:22:23 $
-  Version:   $Revision: 1.19 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmSetSourceFilesPropertiesCommand.h"
 
 #include "cmSourceFile.h"
@@ -33,7 +28,8 @@ bool cmSetSourceFilesPropertiesCommand
   std::vector<std::string>::const_iterator j;
   j = args.begin();
   // old style allows for specifier before PROPERTIES keyword
-  while (*j != "ABSTRACT" &&
+  while (j != args.end() &&
+         *j != "ABSTRACT" &&
          *j != "WRAP_EXCLUDE" &&
          *j != "GENERATED" &&
          *j != "COMPILE_FLAGS" &&
